/* IFV to MFV converter
 *
 * original code from http://www.mikrocontroller.net/topic/311479
 * cleaned up and partly rewritten by Axel XL Schwenke
 *
 * $Id: main.c 217 2013-10-17 18:23:57Z schwenke $
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>


#include "dtmf_tables.h"

/* digit to mfv decoding table
 * high nibble = index in dtmf_freqL
 * low nibble  = index in dtmf_freqH
 */
const unsigned char n2mfv [10] =
{
    // 0     1     2     3     4     5     6     7     8     9
    0x31, 0x00, 0x01, 0x02, 0x10, 0x11, 0x12, 0x20, 0x21, 0x22
};


/* the following code is stolen from AN314 */

volatile unsigned char SWa = 0;      // step width of high frequency
volatile unsigned char SWb = 0;      // step width of low frequency

ISR (TIMER1_OVF_vect)  {
    // position freq. A/B in LUT (extended format)
    static unsigned int CurSinValA = 0;
    static unsigned int CurSinValB = 0;

    // move Pointer about step width aheaed
    CurSinValA += SWa;
    CurSinValB += SWb;

    // normalize Temp-Pointer
    unsigned char TmpSinValA = ((CurSinValA + SHIFT) >> SCALE) % N_SAMPLES;
    unsigned char TmpSinValB = ((CurSinValB + SHIFT) >> SCALE) % N_SAMPLES;

    // get sinus samples from LUT
    unsigned char sampleA = pgm_read_byte(&sin_table[TmpSinValA]);
    unsigned char sampleB = pgm_read_byte(&sin_table[TmpSinValB]);

    // calculate PWM value: high frequency value + 3/4 low frequency value
    OCR1AL = sampleA + sampleB - (sampleB >> 2);
}

/* end of AN314 code */

/* this is a heartbeat counter */
volatile unsigned int takt = 0;
ISR (TIMER0_OVF_vect) {
    takt++;
}

/* Initialization */
void init (void)
{
    DDRB = 0x00;

    OSCCAL = 0x4A;

    TCCR0A =0;
    TCCR0B =  (1<<CS02);
    TCCR1A = (1<<COM1A1)+(1<<WGM10);   // non inverting / 8Bit PWM
    TCCR1B = (1<<CS10);                // CLK/1
    TIMSK  =  (1<<TOIE0);              // Int T1 Overflow enabled

    MCUCR  = (1<<SE);

    sei();
}


/* emit DTMF tone for number z */
void tone(unsigned char z)
{
    /* setup DTMF frequencies */
    unsigned char mfv = n2mfv[z];
    SWa = dtmf_freqH[(mfv)    & 0x03];
    SWb = dtmf_freqL[(mfv>>4) & 0x03];

    /* enable PWM interrupt and IO pin */
    TIMSK |= (1<<TOIE1);
    DDRB  |= (1<<PB3);

    /* wait for 15 heartbeats */
    takt = 0;
    while (takt<15) sleep_cpu();

    /* disable PWM interrupt and IO pin */
    DDRB  &= ~(1<<PB3);
    TIMSK &= ~(1<<TOIE1);
}


int main(void)
{
    // the dialed number
    unsigned char nr = 0;

    init();

    while (takt<30) sleep_cpu();

    while (1) {
        /* wait for PD0 becoming L */
        while (PIND & 0x01) sleep_cpu();

        /* delay 3 */
        takt=0; while (takt<3) sleep_cpu();

        /* wait for PD0 becoming H */
        takt=0;
        while ((!(PIND & 0x01))) sleep_cpu();

        /* check if pulse length is valid */
        if (takt < 30) nr++;
        /* else reset loop */
        else nr = 0;

        /* delay 3 */
        takt=0; while (takt<3) sleep_cpu();

        /* wait for next pulse within 20 clocks */
        takt = 0;
        while ((takt < 20) & (PIND & 0x01)) sleep_cpu();

        /* no pulse after 20 clocks -> dialing complete */
        if (takt>=20) {
            // 0 gives 10 pulses, make sure to wrap over
            tone(nr%10);
            nr = 0;
        }
    }

    /* not reached */
    return 0;
}

